/*
 *	Lame time routines source file
 *
 *	Copyright (c) 2000 Mark Taylor
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: lametime.c,v 1.5 2000/11/18 10:50:56 aleidinger Exp $ */

/*
 * name:        GetCPUTime ( void )
 *
 * description: returns CPU time used by the process
 * input:       none
 * output:      time in seconds
 * known bugs:  may not work in SMP and RPC
 * conforming:  ANSI C
 *
 * There is some old difficult to read code at the end of this file.
 * Can someone integrate this into this function (if useful)?
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#elif defined(HAVE_CONFIG_MS_H)
# include <configMS.h>
#endif

#include <stdio.h>
#include <assert.h>
#include <time.h>

double GetCPUTime ( void )
{
    clock_t  t;

#if defined(_MSC_VER)  ||  defined(__BORLANDC__)    
    t = clock ();
#else
    t = clock ();
#endif    
    return t / (double) CLOCKS_PER_SEC;
}


/*
 * name:        GetRealTime ( void )
 *
 * description: returns real (human) time elapsed relative to a fixed time (mostly 1970-01-01 00:00:00)
 * input:       none
 * output:      time in seconds
 * known bugs:  bad precision with time()
 */

#if defined(__unix__)  ||  defined(SVR4)  ||  defined(BSD)

# include <sys/time.h>
# include <unistd.h>

double GetRealTime ( void )			/* conforming:  SVr4, BSD 4.3 */
{
    struct timeval  t;

    if ( 0 != gettimeofday (&t, NULL) )
        assert (0);
    return t.tv_sec + 1.e-6 * t.tv_usec;
} 

#elif defined(WIN16)  ||  defined(WIN32)

# include <stdio.h>
# include <sys/types.h>
# include <sys/timeb.h>

double GetRealTime ( void )			/* conforming:  Win 95, Win NT */
{
    struct _timeb  t;
    
    _ftime ( &t );
    return t.time + 1.e-3 * t.millitm;
}

#else

double GetRealTime ( void )			/* conforming:  SVr4, SVID, POSIX, X/OPEN, BSD 4.3 */
{						/* BUT NOT GUARANTEED BY ANSI */
    time_t  t;
    
    t = time (NULL);
    return (double) t;
} 

#endif
                              

#ifdef _WIN32
# include <io.h>
# include <fcntl.h>
#else
# include <unistd.h>
#endif

int  lame_set_stream_binary_mode ( FILE* const fp )
{
#if   defined __EMX__
    _fsetmode ( fp, "b" );
#elif defined __BORLANDC__
    setmode   (_fileno(fp),  O_BINARY );
#elif defined __CYGWIN__
    setmode   ( fileno(fp), _O_BINARY );
#elif defined _WIN32
    _setmode  (_fileno(fp), _O_BINARY );
#endif
    return 0;
}


#if defined(__riscos__)
# include <kernel.h>
# include <sys/swis.h>
#elif defined(_WIN32)
# include <sys/types.h>
# include <sys/stat.h>
//# include <unistd.h>
#else
# include <sys/stat.h>
//# include <unistd.h>
#endif

off_t  lame_get_file_size ( const char* const filename )
{
    struct stat       sb;

#ifndef __riscos__
    if ( 0 == stat ( filename, &sb ) )
        return sb.st_size;
#else
    _kernel_swi_regs  reg;
    
    reg.r [0] = 17;
    reg.r [1] = (int) filename;
    _kernel_swi ( OS_File, &reg, &reg );
    if ( reg.r [0] == 1 )
        return (off_t) reg.r [4];
#endif
    return (off_t) -1;
}

/* End of lametime.c */
